#!/usr/bin/env python3

import argparse
import os
import sys
import subprocess
import numpy as np
import pyshellspec3

def read_lc(lfile):
    """
    """
    data = np.loadtxt(lfile, dtype=str)

    names = data[:,0]
    pbands = data[:,1]
    errs = data[:,2].astype('float64')

    return names, pbands, errs

def read_if(ifile):
    """

    :param ifile:
    :return:
    """
    return np.loadtxt(ifile, dtype=str)

# constants
ra = (18 + 50 / 60. + 4.79525 / 3600.)/24.*360.
dec = 33 + 21 / 60. + 45.6100 / 3600.
dir_data = 'data'

def main():
    """
    Main loop
    :return:
    """
    # read the data
    obs = []

    # spectral data
    dir = os.path.join(dir_data, '')
    obs.append(pyshellspec3.SPEData(filename=os.path.join(dir,'Spectra.dat')))

    # interferometric data
    obs.append(pyshellspec3.IFData(filename=os.path.join(dir, 'Vis.dat'), location='chara', ra=ra, dec=dec, format='ascii'))

    # construct data class
    data = pyshellspec3.Data(obs)

    # construct the model
    central = pyshellspec3.CentralObject()
    companion = pyshellspec3.Companion()
    nebula = pyshellspec3.Nebula()
    jet = pyshellspec3.Jet()
    spot = pyshellspec3.Spot()
    envelope = pyshellspec3.Envelope()
    shell = pyshellspec3.Shell()
    orbit = pyshellspec3.Orbit()
    objs = [central, companion, nebula, jet, spot, envelope, shell, orbit]
    model = pyshellspec3.Model(objects=objs)

    # construct the Interface
    itf = pyshellspec3.Interface(model=model, data=data, ncpu=8, image_size=720,
        if_phase_precision=3,
        df_phase_precision=3,
        lc_phase_precision=2,
        sed_phase_precision=2,
        spe_phase_precision=2, 
        if_ew_precision=7,
        df_ew_precision=10,
        lc_ew_precision=9,
        sed_ew_precision=10,
        spe_ew_precision=10,
        shellspec_template="template.in",
        shellspec_abundance=os.path.join(os.getcwd(), "abundances"),
        use_offset=True, use_differential=True,
        exclude_visphi=False, dry_run=False, debug=True)

    # body-frozen grid, line-of-sight grid
    tmp=100.0
    itf.set_parameter('rmdfx1', value=-tmp)
    itf.set_parameter('rmdfx2', value=+tmp)
    itf.set_parameter('rmdx1',  value=-tmp)
    itf.set_parameter('rmdx2',  value=+tmp)

    tmp=25.0
    itf.set_parameter('rmdfy1', value=-tmp)
    itf.set_parameter('rmdfy2', value=+tmp)
    itf.set_parameter('rmdfz1', value=-tmp)
    itf.set_parameter('rmdfz2', value=+tmp)
    itf.set_parameter('rmdy1',  value=-tmp)
    itf.set_parameter('rmdy2',  value=+tmp)
    itf.set_parameter('rmdz1',  value=-tmp)
    itf.set_parameter('rmdz2',  value=+tmp)

    tmp=0.5
    itf.set_parameter('stepfx', value=tmp)
    itf.set_parameter('stepfy', value=tmp)
    itf.set_parameter('stepfz', value=tmp)

    itf.set_parameter('stepx', value=tmp)
    itf.set_parameter('stepy', value=tmp)
    itf.set_parameter('stepz', value=tmp)

    # set fitted parameters
    itf.set_parameter('icomp', value=1)
    itf.set_parameter('inebl', value=1)  # nebula
    itf.set_parameter('itnb', value=3)  # power-law
#    itf.set_parameter('ijet', value=2)
#    itf.set_parameter('ienv', value=3) # evelope Roche contact
#    itf.set_parameter('ishell', value=3)

#    itf.set_parameter('istar', value=0)  # no star
    itf.set_parameter('icomp', value=0)  # no companion
#    itf.set_parameter('inebl', value=0)  # no nebula
    itf.set_parameter('ijet', value=0)  # no jet
    itf.set_parameter('ispot', value=0)  # no spot
    itf.set_parameter('ienv', value=0)  # no envelope
    itf.set_parameter('ishell', value=0)  # no shell

    itf.set_parameter('ichemc', value=1)
    itf.set_parameter('ielnd', value=1)
    itf.set_parameter('ithom', value=1) # scattering
    itf.set_parameter('irayl', value=1)
    itf.set_parameter('imie', value=0)
    itf.set_parameter('imiepf', value=0)
    itf.set_parameter('iline', value=1)
    itf.set_parameter('iinvnb', value=1)  # inversion of T
    itf.set_parameter('ivelnb', value=1)  # radial wind
    itf.set_parameter('ishdnb', value=1)  # shadow

    itf.set_parameter('dd'      , value=100.0                , fitted=False, vmin=305., vmax=330.)
    itf.set_parameter('vgamma'  , value=-18.0                , fitted=False, vmin=-100., vmax=100.)

    R_S = 7.e8
    M_S = 2.e30
    G = 6.67e-11
    d = 86400.
    a = 80.0*R_S
    m1 = 10.0*M_S 
    q = 1.0
    m2 = q*m1
    P = np.sqrt(a**3*4.*np.pi**2/(G*(m1+m2)))
    t0 = 2456451.372100 - P/d/4.
    print("P = ", P/d, " d")

    itf.set_parameter('period'  , value=P/d                  , fitted=False, vmin=0., vmax=1000.)
    itf.set_parameter('t0'      , value=t0                   , fitted=False, vmin=0., vmax=1.0e10)
    itf.set_parameter('qeph'    , value=0.0                  , fitted=False, vmin=0., vmax=1.0e-5)
    itf.set_parameter('q'       , value=q                    , fitted=False, vmin=0.10, vmax=1.0)
    itf.set_parameter('ecc'     , value=0.0                  , fitted=False, vmin=0., vmax=0.5)
    itf.set_parameter('dinc'    , value=90.0                 , fitted=False, vmin=80., vmax=110.)
    itf.set_parameter('omega'   , value=0.0                  , fitted=False, vmin=-180., vmax=180.)
    itf.set_parameter('omega_an', value=90.0                 , fitted=False, vmin=-180., vmax=180.)
    itf.set_parameter('asini'   , value=a/R_S                , fitted=False, vmin=53., vmax=63.)

    itf.set_parameter('emstar'  , value=m1/M_S               , fitted=False, vmin=5.0, vmax=15.0)
    itf.set_parameter('rstar'   , value=5.0                  , fitted=False, vmin=1.0, vmax=20.0)
    itf.set_parameter('tstar'   , value=30000.0              , fitted=False, vmin=20000.0, vmax=40000.0)
    itf.set_parameter('vrotst'  , value=200.0                , fitted=False, vmin=0.0, vmax=1000.0)

    itf.set_parameter('rcp'     , value=5.0                  , fitted=False, vmin=1.0, vmax=20.0)
    itf.set_parameter('tempcp'  , value=30000.0              , fitted=False, vmin=20000.0, vmax=40000.0)
    itf.set_parameter('vrotcp'  , value=200.0                , fitted=False, vmin=0.0, vmax=1000.0)

    itf.set_parameter('rinnb'   , value=6.0                  , fitted=False, vmin=6.0, vmax=12.0)
    itf.set_parameter('routnb'  , value=9.0                  , fitted=False, vmin=10.0, vmax=35.5)
    itf.set_parameter('hinvnb'  , value=5.0                  , fitted=False, vmin=1.0, vmax=5.0)
    itf.set_parameter('tinvnb'  , value=1.0                  , fitted=False, vmin=1.0, vmax=2.0)
    itf.set_parameter('hwindnb' , value=5.0                  , fitted=False, vmin=3.0, vmax=5.0)
    itf.set_parameter('hcnb'    , value=10.0                 , fitted=False, vmin=1.0, vmax=15.0)
    itf.set_parameter('hvelnb'  , value=5.0                  , fitted=False, vmin=0.1, vmax=5.0)
    itf.set_parameter('vnb'     , value=0.0                  , fitted=False, vmin=0.0, vmax=200.0)
    itf.set_parameter('evelnb'  , value=0.0                  , fitted=False, vmin=0.0, vmax=2.0)
    itf.set_parameter('hshdnb'  , value=5.0                  , fitted=False, vmin=1.0, vmax=5.0)
    itf.set_parameter('tempnb'  , value=30000.0              , fitted=False, vmin=23000., vmax=34000.)
    itf.set_parameter('densnb'  , value=2.0e-11              , fitted=False, vmin=1e-12, vmax=5e-9)
    itf.set_parameter('vtrbnb'  , value=0.0                  , fitted=False, vmin=0.0, vmax=100.0)
    itf.set_parameter('edennb'  , value=-1.0                 , fitted=False, vmin=-3.0, vmax=-0.5)
    itf.set_parameter('etmpnb'  , value=-1.0                 , fitted=False, vmin=-3.0, vmax=-0.5)
    itf.set_parameter('aneb'    , value=5.0                  , fitted=False, vmin=3.0, vmax=12.0)

    # compute one/fit
    itf.compute_chi2(verbose=True)

#    itf.run_fit(fitter='sp_diff_evol', tol=1e-2, maxiter=10000)
#    itf.run_fit(fitter='nlopt_nelder_mead', ftol=1e-6, maxiter=10000)
#    itf.run_fit(fitter='nlopt_sbplx', ftol=1e-3, maxiter=10000)

    itf.write_iterations()
    itf.set_model_to_shellspec()
    itf.write_template('final.in')
    itf.write_model()

    print("Note: fit.py ended successfully.")

    subprocess.check_output("./synthetic_image.plt", shell=True)
    subprocess.check_output("./visibility.plt", shell=True)
    sys.exit(0) 

if __name__ == '__main__':
    main()


